"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ConfiguratioDbSource = void 0;
/* eslint-disable @typescript-eslint/no-unsafe-argument */
/* eslint-disable @typescript-eslint/no-unsafe-call */
/* eslint-disable @typescript-eslint/no-unsafe-assignment */
/* eslint-disable security/detect-object-injection */
/* eslint-disable @typescript-eslint/no-unsafe-member-access */
const internal_logger_1 = require("@spinajs/internal-logger");
/* eslint-disable prettier/prettier */
const configuration_common_1 = require("@spinajs/configuration-common");
const di_1 = require("@spinajs/di");
const orm_1 = require("@spinajs/orm");
const lodash_1 = __importDefault(require("lodash"));
const DbConfig_js_1 = require("./models/DbConfig.js");
let ConfiguratioDbSource = class ConfiguratioDbSource extends configuration_common_1.ConfigurationSource {
    get Order() {
        // load as last, we want to have access to
        // connection options
        return 999;
    }
    async Load(configuration) {
        this.Configuration = configuration;
        this.Options = this.Configuration.get('configuration_db_source', {
            connection: 'default',
            table: 'configuration',
        });
        await this.Connect();
        if ((await this.CheckTable()) === false) {
            internal_logger_1.InternalLogger.warn(`Table for db configuration source not exists. Please run migration before use !`, 'configuration-db-source');
            return null;
        }
        const final = await this.LoadConfigurationFromDB();
        internal_logger_1.InternalLogger.success(`Configuration merged`, 'Configuration-db-source');
        return final;
    }
    async LoadConfigurationFromDB() {
        const dbOptions = await this.Connection.select().from(this.Options.table);
        dbOptions.forEach((entry) => {
            entry.Value = (0, DbConfig_js_1.parse)(entry.Value, entry.Type);
        });
        const grouped = lodash_1.default.groupBy(dbOptions, 'Group');
        const final = {
            onConfigLoad: null,
        };
        for (const k in grouped) {
            final[k] = {};
            for (const v of grouped[k]) {
                // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
                final[k][v.Slug] = v.Value;
            }
        }
        return final;
    }
    async CheckTable() {
        return await this.Connection.schema().tableExists(this.Options.table);
    }
    async Connect() {
        // we use raw connection instead of ORM,
        // ORM requires configuration module to load first
        const dbConnections = this.Configuration.get('db.Connections');
        if (!dbConnections) {
            throw new Error(`db.Connections configuration is not set, please check your config files or set proper db connection options`);
        }
        const dbConnection = this.Options.connection === 'default' ? this.Configuration.get('db.DefaultConnection', this.Options.connection) : this.Options.connection;
        const cfgConnectionOptions = dbConnections.find((x) => x.Name === dbConnection);
        if (!cfgConnectionOptions) {
            throw new Error(`Connection for configuration-db-source named ${dbConnection} not exists, please check your default connection name or if ${dbConnection} exists in configuration.`);
        }
        internal_logger_1.InternalLogger.trace(`Using db connection ${dbConnection}`, 'Configuration-db-source');
        // create or get connection
        if (di_1.DI.has(orm_1.Orm)) {
            this.Connection = di_1.DI.get(orm_1.Orm).Connections.get(dbConnection);
        }
        else {
            this.Connection = di_1.DI.resolve(cfgConnectionOptions.Driver, [cfgConnectionOptions]);
            await this.Connection.connect();
        }
    }
};
exports.ConfiguratioDbSource = ConfiguratioDbSource;
exports.ConfiguratioDbSource = ConfiguratioDbSource = __decorate([
    (0, di_1.Singleton)(),
    (0, di_1.Injectable)(configuration_common_1.ConfigurationSource)
], ConfiguratioDbSource);
//# sourceMappingURL=source.js.map