var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
/* eslint-disable @typescript-eslint/no-explicit-any */
/* eslint-disable @typescript-eslint/no-unsafe-argument */
/* eslint-disable @typescript-eslint/no-unsafe-call */
/* eslint-disable @typescript-eslint/no-unsafe-member-access */
/* eslint-disable @typescript-eslint/require-await */
import { Injectable, Bootstrapper, DI } from '@spinajs/di';
import { DbConfig } from './models/DbConfig.js';
import CONFIGURATION_SCHEMA from './schemas/configuration.db.source.schema.js';
import { Configuration } from '@spinajs/configuration-common';
import { InsertBehaviour } from '@spinajs/orm';
import _ from 'lodash';
/**
 * watch interval, default 3 min
 */
const CONFIG_WATCH_TIMER_INTERVAL = 3 * 60 * 1000;
function __saveConfigOptions(v) {
    if (v.options.expose) {
        void DbConfig.insert({
            Slug: v.path,
            Value: v.options.defaultValue,
            Group: v.options.exposeOptions?.group,
            Label: v.options.exposeOptions?.label,
            Description: v.options.exposeOptions?.description,
            Meta: v.options.exposeOptions?.meta,
            Required: v.options.required,
            Type: v.options.exposeOptions?.type,
            Watch: v.options.exposeOptions?.watch ?? false,
            Default: v.options.defaultValue ?? undefined,
            Exposed: true,
        }, InsertBehaviour.InsertOrIgnore);
    }
}
let DbConfigSourceBotstrapper = class DbConfigSourceBotstrapper extends Bootstrapper {
    async bootstrap() {
        DI.register(CONFIGURATION_SCHEMA).asValue('__configurationSchema__');
        // register all conf options
        // as we go ( eg. resolved service )
        DI.on('di.registered.__configuration_property__', (v) => {
            if (v.options && v.options.expose) {
                __saveConfigOptions(v);
                void DbConfig.where("Slug", v.path).first().then((result) => {
                    const cService = DI.get(Configuration);
                    cService.set(v.path, result?.Value ?? v.options.defaultValue);
                });
            }
        });
        // register vals added before orm is resolved eg. at bostrap phase
        DI.once('di.resolved.Orm', (container) => {
            const vars = container
                .get(Array.ofType('__configuration_property__'))
                .filter((x) => x.options)
                .filter((x) => x.options.expose);
            /**
             * Insert to db all exposed config options
             */
            for (const v of vars) {
                __saveConfigOptions(v);
            }
            const interval = DI.get('__config_watch_interval__');
            const watchTimer = setInterval(() => {
                const varsToWatch = vars.filter((x) => x.options.exposeOptions.watch);
                const cService = container.get(Configuration);
                if (varsToWatch.length === 0) {
                    return;
                }
                void DbConfig.query()
                    .whereIn('Slug', varsToWatch.map((x) => x.path))
                    .then((result) => {
                    result.forEach((r) => {
                        const cfgVal = cService.get(`${r.Group}.${r.Slug}`);
                        if (!_.isEqual(cfgVal, r.Value)) {
                            cService.set(`${r.Group}.${r.Slug}`, r.Value);
                        }
                    });
                    return;
                });
            }, interval?.value || CONFIG_WATCH_TIMER_INTERVAL);
            DI.once('di.dispose', () => {
                clearInterval(watchTimer);
            });
        });
        return;
    }
};
DbConfigSourceBotstrapper = __decorate([
    Injectable(Bootstrapper)
], DbConfigSourceBotstrapper);
export { DbConfigSourceBotstrapper };
//# sourceMappingURL=bootstrap.js.map