var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
/* eslint-disable @typescript-eslint/no-unsafe-argument */
/* eslint-disable @typescript-eslint/no-unsafe-call */
/* eslint-disable @typescript-eslint/no-unsafe-assignment */
/* eslint-disable security/detect-object-injection */
/* eslint-disable @typescript-eslint/no-unsafe-member-access */
import { InternalLogger } from '@spinajs/internal-logger';
/* eslint-disable prettier/prettier */
import { ConfigurationSource } from '@spinajs/configuration-common';
import { DI, Injectable, Singleton } from '@spinajs/di';
import { Orm } from '@spinajs/orm';
import _ from 'lodash';
import { parse } from './models/DbConfig.js';
let ConfiguratioDbSource = class ConfiguratioDbSource extends ConfigurationSource {
    get Order() {
        // load as last, we want to have access to
        // connection options
        return 999;
    }
    async Load(configuration) {
        this.Configuration = configuration;
        this.Options = this.Configuration.get('configuration_db_source', {
            connection: 'default',
            table: 'configuration',
        });
        await this.Connect();
        if ((await this.CheckTable()) === false) {
            InternalLogger.warn(`Table for db configuration source not exists. Please run migration before use !`, 'configuration-db-source');
            return null;
        }
        const final = await this.LoadConfigurationFromDB();
        InternalLogger.success(`Configuration merged`, 'Configuration-db-source');
        return final;
    }
    async LoadConfigurationFromDB() {
        const dbOptions = await this.Connection.select().from(this.Options.table);
        dbOptions.forEach((entry) => {
            entry.Value = parse(entry.Value, entry.Type);
        });
        const grouped = _.groupBy(dbOptions, 'Group');
        const final = {
            onConfigLoad: null,
        };
        for (const k in grouped) {
            final[k] = {};
            for (const v of grouped[k]) {
                // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
                final[k][v.Slug] = v.Value;
            }
        }
        return final;
    }
    async CheckTable() {
        return await this.Connection.schema().tableExists(this.Options.table);
    }
    async Connect() {
        // we use raw connection instead of ORM,
        // ORM requires configuration module to load first
        const dbConnections = this.Configuration.get('db.Connections');
        if (!dbConnections) {
            throw new Error(`db.Connections configuration is not set, please check your config files or set proper db connection options`);
        }
        const dbConnection = this.Options.connection === 'default' ? this.Configuration.get('db.DefaultConnection', this.Options.connection) : this.Options.connection;
        const cfgConnectionOptions = dbConnections.find((x) => x.Name === dbConnection);
        if (!cfgConnectionOptions) {
            throw new Error(`Connection for configuration-db-source named ${dbConnection} not exists, please check your default connection name or if ${dbConnection} exists in configuration.`);
        }
        InternalLogger.trace(`Using db connection ${dbConnection}`, 'Configuration-db-source');
        // create or get connection
        if (DI.has(Orm)) {
            this.Connection = DI.get(Orm).Connections.get(dbConnection);
        }
        else {
            this.Connection = DI.resolve(cfgConnectionOptions.Driver, [cfgConnectionOptions]);
            await this.Connection.connect();
        }
    }
};
ConfiguratioDbSource = __decorate([
    Singleton(),
    Injectable(ConfigurationSource)
], ConfiguratioDbSource);
export { ConfiguratioDbSource };
//# sourceMappingURL=source.js.map